package com.diagbot.aggregate;

import com.diagbot.dto.PushBaseDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.dto.PushScaleDTO;
import com.diagbot.enums.ConceptTypeEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.facade.MappingConfigFacade;
import com.diagbot.util.ListUtil;
import com.google.common.collect.Lists;
import io.github.lvyahui8.spring.annotation.DataConsumer;
import io.github.lvyahui8.spring.annotation.DataProvider;
import io.github.lvyahui8.spring.annotation.InvokeParameter;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description:
 * @author: gaodm
 * @time: 2020/8/19 13:50
 */
@Component
public class AssemblePushAggregate {

    @Autowired
    private MappingConfigFacade mappingConfigFacade;

    @DataProvider("assemblePushAll")
    public PushDTO assemblePushAll(
            @InvokeParameter("pushDTO") PushDTO data,
            @InvokeParameter("hospitalId") Long hospitalId,
            @DataConsumer("retLis") List<PushBaseDTO> retLis,
            @DataConsumer("retPacs") List<PushBaseDTO> retPacs,
            @DataConsumer("retDiagMap") Map<String, List<PushBaseDTO>> retDiagMap,
            @DataConsumer("retOperation") List<PushBaseDTO> retOperation,
            @DataConsumer("retDrug") List<PushBaseDTO> retDrug,
            @DataConsumer("retDrugs") Map<String, List<PushBaseDTO>> retDrugs,
            @DataConsumer("retScale") List<PushScaleDTO> retScale,
            @DataConsumer("retNurse") List<PushBaseDTO> retNurse) {
        //检验
        if (ListUtil.isNotEmpty(retLis)) {
            data.setLis(retLis);
        } else {
            data.setOperations(null);
        }
        //检查
        if (ListUtil.isNotEmpty(retPacs)) {
            data.setPacs(retPacs);
        } else {
            data.setOperations(null);
        }
        //诊断
        if (MapUtils.isNotEmpty(retDiagMap)) {
            data.setDis(retDiagMap);
        } else {
            data.setOperations(null);
        }
        //手术
        if (ListUtil.isNotEmpty(retOperation)) {
            data.setOperations(retOperation);
        } else {
            data.setOperations(null);
        }
        //药品
        if (ListUtil.isNotEmpty(retDrug)) {
            data.setMedicines(retDrug);
        }
        //分类药品
        if (MapUtils.isNotEmpty(retDrugs)) {
            data.setMedicine(retDrugs);
        } else {
            data.setOperations(null);
        }
        //量表
        if (ListUtil.isNotEmpty(retScale)) {
            data.setScale(retScale);
        }
        //护理
        if (ListUtil.isNotEmpty(retNurse)) {
            data.setNurse(retNurse);
        }
        return data;
    }

    @DataProvider("retLis")
    public List<PushBaseDTO> retLis(@InvokeParameter("pushDTO") PushDTO data,
                                    @InvokeParameter("hospitalId") Long hospitalId) {
        //检验
        List<PushBaseDTO> retLis = new ArrayList<>();
        //检验出参
        List<PushBaseDTO> retOutLis = new ArrayList<>();
        Map<String, String> stringMap = new HashMap<>();
        if (ListUtil.isNotEmpty(data.getLis())) {
            retLis = data.getLis();
            List<String> uniqueNameList = retLis.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.LisPack.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (hospitalId == -1) {
                return retLis;
            }

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {

                for (Map.Entry<String, Map<String, Map<String, List<String>>>> uniqueNameMapEntry : uniqueNameMap.entrySet()) {
                    if (uniqueNameMapEntry.getValue() != null) {
                        for (Map.Entry<String, Map<String, List<String>>> entry : uniqueNameMapEntry.getValue().entrySet()) {
                            stringMap.put(uniqueNameMapEntry.getKey(), entry.getKey());
                        }
                    }
                }

                for(PushBaseDTO pushDto : retLis){
                    for (Map.Entry<String, String> stringEntry : stringMap.entrySet()) {
                        if (stringEntry.getValue().equals(pushDto.getName())) {
                            PushBaseDTO pushBaseDTO = new PushBaseDTO();
                            pushBaseDTO.setName(stringEntry.getKey());
                            retOutLis.add(pushBaseDTO);
                        }
                    }
                }

//                retLis.forEach(item -> {
//                    if (uniqueNameMap.get(item.getName()) != null) {
//                        item.setHisNameList(uniqueNameMap.get(item.getName())
//                                .values().stream()
//                                .map(i -> i.keySet())
//                                .flatMap(Collection::stream)
//                                .collect(Collectors.toList()));
//                    }
//                });
            }
        }
        return retOutLis;
    }

    @DataProvider("retPacs")
    public List<PushBaseDTO> retPacs(@InvokeParameter("pushDTO") PushDTO data,
                                     @InvokeParameter("hospitalId") Long hospitalId) {
        //检查
        List<PushBaseDTO> retPacs = new ArrayList<>();
        //检查出参
        List<PushBaseDTO> retOutPacs = new ArrayList<>();
        Map<String, String> stringMap = new HashMap<>();
        if (ListUtil.isNotEmpty(data.getPacs())) {
            retPacs = data.getPacs();
            List<String> uniqueNameList = retPacs.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Pacs.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (hospitalId == -1) {
                return retPacs;
            }

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
//                // 遍历外层 Map
//                for (String key : uniqueNameMap.keySet()) {
//                    // 创建 PushBaseDTO 对象并设置 name 字段
//                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
//                    pushBaseDTO.setName(key);
//                    retOutPacs.add(pushBaseDTO);
//                }
                for (Map.Entry<String, Map<String, Map<String, List<String>>>> uniqueNameMapEntry : uniqueNameMap.entrySet()) {
                    if (uniqueNameMapEntry.getValue() != null) {
                        for (Map.Entry<String, Map<String, List<String>>> entry : uniqueNameMapEntry.getValue().entrySet()) {
                            stringMap.put(uniqueNameMapEntry.getKey(), entry.getKey());
                        }
                    }
                }

                for(PushBaseDTO pushDto : retPacs){
                    for (Map.Entry<String, String> stringEntry : stringMap.entrySet()) {
                        if (stringEntry.getValue().equals(pushDto.getName())) {
                            PushBaseDTO pushBaseDTO = new PushBaseDTO();
                            pushBaseDTO.setName(stringEntry.getKey());
                            retOutPacs.add(pushBaseDTO);
                        }
                    }
                }
//                retPacs.forEach(item -> {
//                    if (uniqueNameMap.get(item.getName()) != null) {
//                        item.setHisNameList(uniqueNameMap.get(item.getName())
//                                .values().stream()
//                                .map(i -> i.keySet())
//                                .flatMap(Collection::stream)
//                                .collect(Collectors.toList()));
//                    }
//                });
            }
        }
        return retOutPacs;
    }

    @DataProvider("retDiagMap")
    public Map<String, List<PushBaseDTO>> retDiagMap(@InvokeParameter("pushDTO") PushDTO data,
                                                     @InvokeParameter("hospitalId") Long hospitalId) {
        Map<String, List<PushBaseDTO>> retMap = new HashMap<>();
        //出参
        Map<String, List<PushBaseDTO>> retOutDiags = new LinkedHashMap<>();
        if (data.getDis() != null && data.getDis().size() > 0) {
            retMap = data.getDis();
            List<String> uniqueNameList = Lists.newLinkedList();
            for (Map.Entry<String, List<PushBaseDTO>> entry : retMap.entrySet()) {
                if (ListUtil.isNotEmpty(entry.getValue())) {
                    uniqueNameList.addAll(entry.getValue().stream()
                            .map(i -> i.getName())
                            .collect(Collectors.toList()));
                }
            }
            if (ListUtil.isNotEmpty(uniqueNameList)) {
                Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                        = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Disease.getKey(), hospitalId, StatusEnum.Enable.getKey());

                if (hospitalId == -1) {
                    return retMap;
                }

                List<PushBaseDTO> diagList = new ArrayList<>();
                for (Map.Entry<String, List<PushBaseDTO>> entry : retMap.entrySet()) {
                    if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                        // 遍历外层 Map
                        for (String key : uniqueNameMap.keySet()) {
                            // 创建 PushBaseDTO 对象并设置 name 字段
                            PushBaseDTO pushBaseDTO = new PushBaseDTO();
                            pushBaseDTO.setName(key);
                            diagList.add(pushBaseDTO);
                        }
                    }
                    retOutDiags.put(entry.getKey(), diagList);
                }
//                for (Map.Entry<String, List<PushBaseDTO>> entry : retMap.entrySet()) {
//                    if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
//                        entry.getValue().forEach(item -> {
//                            if (uniqueNameMap.get(item.getName()) != null) {
//                                item.setHisNameList(uniqueNameMap.get(item.getName())
//                                        .values().stream()
//                                        .map(i -> i.keySet())
//                                        .flatMap(Collection::stream)
//                                        .collect(Collectors.toList()));
//                            }
//                        });
//                    }
//                }
            }
        }
        return retOutDiags;
    }

    @DataProvider("retOperation")
    public List<PushBaseDTO> retOperation(@InvokeParameter("pushDTO") PushDTO data,
                                          @InvokeParameter("hospitalId") Long hospitalId) {
        //手术
        List<PushBaseDTO> retOperation = new ArrayList<>();
        //手术出参
        List<PushBaseDTO> retOutOperation = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getOperations())) {
            retOperation = data.getOperations();
            List<String> uniqueNameList = retOperation.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Operation.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (hospitalId == -1) {
                return retOperation;
            }
            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                // 遍历外层 Map
                for (String key : uniqueNameMap.keySet()) {
                    // 创建 PushBaseDTO 对象并设置 name 字段
                    PushBaseDTO pushBaseDTO = new PushBaseDTO();
                    pushBaseDTO.setName(key);
                    retOutOperation.add(pushBaseDTO);
                }
//                retOperation.forEach(item -> {
//                    if (uniqueNameMap.get(item.getName()) != null) {
//                        item.setHisNameList(uniqueNameMap.get(item.getName())
//                                .values().stream()
//                                .map(i -> i.keySet())
//                                .flatMap(Collection::stream)
//                                .collect(Collectors.toList()));
//                    }
//                });
            }
        }
        return retOutOperation;
    }

    @DataProvider("retDrug")
    public List<PushBaseDTO> retDrug(@InvokeParameter("pushDTO") PushDTO data,
                                     @InvokeParameter("hospitalId") Long hospitalId) {
        //药品
        List<PushBaseDTO> retDrug = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getMedicines())) {
            retDrug = data.getMedicines();
            List<String> uniqueNameList = retDrug.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Drug.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retDrug.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retDrug;
    }

    @DataProvider("retDrugs")
    public Map<String, List<PushBaseDTO>> retDrugs(@InvokeParameter("pushDTO") PushDTO data,
                                                   @InvokeParameter("hospitalId") Long hospitalId) {
        //药品
        Map<String, List<PushBaseDTO>> retDrugs = new LinkedHashMap<>();
        //出参
        Map<String, List<PushBaseDTO>> retOutDrugs = new LinkedHashMap<>();
        if (MapUtils.isNotEmpty(data.getMedicine())) {
            retDrugs = data.getMedicine();
            for (Map.Entry<String, List<PushBaseDTO>> entry : retDrugs.entrySet()) {
                List<String> uniqueNameList = entry.getValue().stream()
                        .map(i -> i.getName())
                        .collect(Collectors.toList());
                Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                        = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Drug.getKey(), hospitalId, StatusEnum.Enable.getKey());

                if (hospitalId == -1) {
                    return retDrugs;
                }

                List<PushBaseDTO> drugList = new ArrayList<>();
                if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                    // 遍历外层 Map
                    for (String key : uniqueNameMap.keySet()) {
                        // 创建 PushBaseDTO 对象并设置 name 字段
                        PushBaseDTO pushBaseDTO = new PushBaseDTO();
                        pushBaseDTO.setName(key);
                        drugList.add(pushBaseDTO);
                    }
                    retOutDrugs.put(entry.getKey(), drugList);
//                    entry.getValue().forEach(item -> {
//                        if (uniqueNameMap.get(item.getName()) != null) {
//                            item.setHisNameList(uniqueNameMap.get(item.getName())
//                                    .values().stream()
//                                    .map(i -> i.keySet())
//                                    .flatMap(Collection::stream)
//                                    .collect(Collectors.toList()));
//                        }
//                    });
                }
            }
        }
        return retOutDrugs;
    }

    @DataProvider("retScale")
    public List<PushScaleDTO> retScale(@InvokeParameter("pushDTO") PushDTO data,
                                       @InvokeParameter("hospitalId") Long hospitalId) {
        //量表
        List<PushScaleDTO> retScale = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getScale())) {
            retScale = data.getScale();
            List<String> uniqueNameList = retScale.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Scale.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retScale.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retScale;
    }

    @DataProvider("retNurse")
    public List<PushBaseDTO> retNurse(@InvokeParameter("pushDTO") PushDTO data,
                                      @InvokeParameter("hospitalId") Long hospitalId) {
        //量表
        List<PushBaseDTO> retNurse = new ArrayList<>();
        if (ListUtil.isNotEmpty(data.getNurse())) {
            retNurse = data.getNurse();
            List<String> uniqueNameList = retNurse.stream()
                    .map(i -> i.getName())
                    .collect(Collectors.toList());
            Map<String, Map<String, Map<String, List<String>>>> uniqueNameMap
                    = mappingConfigFacade.groupByUniqueNameWithName(uniqueNameList, ConceptTypeEnum.Nurse.getKey(), hospitalId, StatusEnum.Enable.getKey());

            if (uniqueNameMap != null && uniqueNameMap.size() > 0) {
                retNurse.forEach(item -> {
                    if (uniqueNameMap.get(item.getName()) != null) {
                        item.setHisNameList(uniqueNameMap.get(item.getName())
                                .values().stream()
                                .map(i -> i.keySet())
                                .flatMap(Collection::stream)
                                .collect(Collectors.toList()));
                    }
                });
            }
        }
        return retNurse;
    }
}