package com.diagbot.facade;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.*;
import com.diagbot.entity.TemplateInfo;
import com.diagbot.entity.TranPacsInfo;
import com.diagbot.entity.TranDatadockingLog;
import com.diagbot.entity.TranHospitalRelation;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.service.impl.TranPacsInfoServiceImpl;
import com.diagbot.util.*;
import com.diagbot.vo.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.stream.Collectors;

/*
 * @Description:对外对接业务处理类
 * @Author:liuqq
 * @time: 2020/11/04 14:51
 **/
@Component
@Slf4j
public class DataPageDockFacade {

    @Autowired
    private MrFacade mrFacade;

    @Autowired
    private PushFacade pushFacade;

    @Autowired
    private DataDockingLogFacade dataDockingLogFacade;

    @Autowired
    private TranHospitalRelationFacade tranHospitalRelationFacade;

    @Value("${data-page.url}")
    private String dataPageUrl;

    @Autowired
    private KlConceptStaticFacade KlConceptStaticFacade;

    @Autowired
    TranPacsInfoServiceImpl tranPacsInfoServiceImpl;


    /**
     * @Description:静态知识开单合理性
     * @Author:liuqq
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO<String> getStaticKnowledge(HisStaticKnowledgeVO hisStaticKnowledgeVO) {
        String url = "";
        //基层子医院业务逻辑
        if (StringUtil.isNotEmpty(hisStaticKnowledgeVO.getHospitalCode()) && StringUtil.isNotEmpty(hisStaticKnowledgeVO.getHospitalName())) {
            //查找基层医院
            Long hospitalId = getHospitalRelation(hisStaticKnowledgeVO.getHospitalCode(), hisStaticKnowledgeVO.getHospitalName());
            if (hospitalId != null) {
                hisStaticKnowledgeVO.setHospitalId(hospitalId);
            } else {
                return RespDTO.onError("子医院信息异常");
            }
        } else if (hisStaticKnowledgeVO.getHospitalId() == null) {
            return RespDTO.onError("请输入医院id");
        }
        //初始日志记录信息
        hisStaticKnowledgeVO.setSubHospitalCode(hisStaticKnowledgeVO.getHospitalCode());
        hisStaticKnowledgeVO.setSubHospitalName(hisStaticKnowledgeVO.getHospitalName());
        //hisStaticKnowledgeVO.setSceneName();
        //判断是否有静态知识
        StaticKnowledgeHISVO staticKnowledgeHISVO = new StaticKnowledgeHISVO();
        BeanUtils.copyProperties(hisStaticKnowledgeVO, staticKnowledgeHISVO);
        Boolean flag = KlConceptStaticFacade.isExistForHIS(staticKnowledgeHISVO);
        //如果有返回静态知识url地址
        if (flag) {
            url = dataPageUrl + "/informationOut.html?hospitalId=" + hisStaticKnowledgeVO.getHospitalId()
                    + "&type=" + hisStaticKnowledgeVO.getType()
                    + "&hisName=" + hisStaticKnowledgeVO.getHisName();
            if (StringUtil.isNotEmpty(hisStaticKnowledgeVO.getHisDetailName())) {
                url += "&hisDetailName=" + hisStaticKnowledgeVO.getHisDetailName();
            }
            return RespDTO.onSuc(url);
        } else {
            return RespDTO.onError("无静态知识");
        }
    }

    /**
     * @Description:页面推送模式
     * @Author:liuqq
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO<String> getDataService(DataPageDockVO dataPageDockVO) {
        if (StringUtil.isNotEmpty(dataPageDockVO.getHospitalCode()) && StringUtil.isNotEmpty(dataPageDockVO.getHospitalName())) {
            //基层子医院业务逻辑
            Long hospitalId = getHospitalRelation(dataPageDockVO.getHospitalCode(), dataPageDockVO.getHospitalName());
            if (hospitalId != null) {
                dataPageDockVO.setHospitalId(hospitalId);
            } else {
                return RespDTO.onError("子医院信息异常");
            }
        }
        if (dataPageDockVO.getHospitalId() == null) {
            return RespDTO.onError("请输入医院id");
        } else if ("".equals(dataPageDockVO.getShowType())) {
            return RespDTO.onError("请输入展示类型,横版：ver，或竖版hor");
        } else if ("".equals(dataPageDockVO.getPlanCode())) {
            return RespDTO.onError("请输入方案编码");
        }

        //初始日志记录信息
        dataPageDockVO.setSubHospitalCode(dataPageDockVO.getHospitalCode());
        dataPageDockVO.setSubHospitalName(dataPageDockVO.getHospitalName());
        dataPageDockVO.setSceneName(dataPageDockVO.getPlanCode());

        String url = "";
        String pushMrId = "";
        PushJoinVO pushJoinVO = new PushJoinVO();
        BeanUtil.copyProperties(dataPageDockVO, pushJoinVO);
        // 将基本参数保存到redis
        String mrId = mrFacade.createMr(pushJoinVO);
        //开单合理性推理时，验证下是否有推送，无推送时，不给url,code为“-1”
        if (dataPageDockVO.getPlanCode().equals("order_rational") || dataPageDockVO.getPlanCode().equals("reference_pacs") || dataPageDockVO.getPlanCode().equals("reference_lis")) {
            IndicationPushVO indicationPushVO = new IndicationPushVO();
            //初始化开单合理性入参
            BeanUtil.copyProperties(dataPageDockVO, indicationPushVO);
            //ruleType（1:危急值提醒，2:开单合理项，3:高危药品、手术，4:其他提醒）
            indicationPushVO.setRuleType("2,3,4");
            //日志中增加入参，将开单合理性参数进行拼接记录
            //CdssLogUtil.addBizReq(indicationPushVO);
            //调用开单合理性接口
            IndicationDTO indicationDTO = pushFacade.indicationPush(indicationPushVO);
            if (indicationDTO != null) {
                if (ListUtil.isNotEmpty(indicationDTO.getBillMsgList()) || ListUtil.isNotEmpty(indicationDTO.getHighRiskList())
                        || ListUtil.isNotEmpty(indicationDTO.getCriticalValList()) || ListUtil.isNotEmpty(indicationDTO.getOtherList())) {
                    pushMrId = mrFacade.createIndicationMr(indicationDTO);
                } else {
                    return RespDTO.onError("无开单合理性提醒！");
                }
            } else {
                return RespDTO.onError("无开单合理性提醒！");
            }
            //将开单合理性结果进行拼接记录
            //CdssLogUtil.addBizResp(indicationPushVO);
        }

        if ("hor".equals(dataPageDockVO.getShowType())) {
            url = dataPageUrl + "/cdssHorizontal.html?pushMrId=" + pushMrId + "&mrId=" + mrId + "&hospitalId=" + dataPageDockVO.getHospitalId() + "&planCode=" + dataPageDockVO.getPlanCode();
        } else {
            url = dataPageUrl + "/cdss.html?pushMrId=" + pushMrId + "&mrId=" + mrId + "&hospitalId=" + dataPageDockVO.getHospitalId() + "&planCode=" + dataPageDockVO.getPlanCode();
        }

        try {
            TranDatadockingLog tranDatadockingLog = new TranDatadockingLog();
            tranDatadockingLog.setHospitalId(dataPageDockVO.getHospitalId());
            tranDatadockingLog.setParamIn(JSON.toJSONString(dataPageDockVO));
            tranDatadockingLog.setParamOut(url);
            tranDatadockingLog.setRemark("页面推送模式，" + dataPageDockVO.getPlanCode());

            tranDatadockingLog.setTransTime(new Date());

            dataDockingLogFacade.save(tranDatadockingLog);
        } catch (Exception e) {
            log.error("页面推送模式，增加日志异常", e.getMessage());
        }

        return RespDTO.onSuc(url);
    }

    /**
     * @Description:解析XML文本（重庆石柱）
     * @Author:dsYun
     * @time: ${DATE} ${TIME}
     **/
    private void analyzeXml(DataPageDockVO dataPageDockVO, String xmlText) {
        Map<String, String> sourceMap = XmlUtil.getXmlToMap(xmlText);
        dataPageDockVO.setSex("男".equals(sourceMap.get("性别")) ? 1 : 2);
        dataPageDockVO.setAge(sourceMap.get("年龄"));
        dataPageDockVO.setPatientId(sourceMap.get("门诊号"));
        dataPageDockVO.setChief(sourceMap.get("主诉"));
        dataPageDockVO.setPasts(sourceMap.get("过去史"));
        dataPageDockVO.setSymptom(sourceMap.get("现病史"));
        dataPageDockVO.setVital(sourceMap.get("查体"));
//        dataPageDockVO.setDiag(getItem(sourceMap.get("诊断")));
//        dataPageDockVO.setDrugOrder(getDrug(sourceMap.get("药品")));
//        dataPageDockVO.setDiseaseName(getDisease(sourceMap.get("诊断")));
//        dataPageDockVO.setOtherAdvice(getItem(sourceMap.get("医嘱")));

        // 查询检查表
        if (ListUtil.isNotEmpty(dataPageDockVO.getApplyNo())) {
            List<TranPacsInfo> pacsInfoList = tranPacsInfoServiceImpl.list(new QueryWrapper<TranPacsInfo>()
                    .eq("is_deleted", IsDeleteEnum.N.getKey())
                    .in("apply_no", dataPageDockVO.getApplyNo())
            );
            if (!pacsInfoList.isEmpty()) {
                dataPageDockVO.setPacsOrder(getPacs(pacsInfoList));
            }
        }
    }

//    private List<Drug> getDrug(String value) {
//        Drug drug = new Drug();
//        List<Drug> drugList = new ArrayList<>();
//        drug.setName(value);
//        //       drug.setUniqueName(value);
//        drugList.add(drug);
//        return drugList;
//    }

//    private List<Item> getItem(String value) {
//        Item item = new Item();
//        List<Item> itemList = new ArrayList<>();
//        item.setName(value);
//        //    item.setUniqueName(value);
//        itemList.add(item);
//        return itemList;
//    }


//    private Item getDisease(String value) {
//        Item item = new Item();
//        item.setName(value);
//        // item.setUniqueName(value);
//        //       item.add(item);
//        return item;
//    }

    private List<Pacs> getPacs(List<TranPacsInfo> pacsInfoList) {
        List<Pacs> pacsList = new ArrayList<>();
        for (TranPacsInfo pacsInfo : pacsInfoList) {
            Pacs pacs = new Pacs();
            if (StringUtil.isNotBlank(pacsInfo.getPacsName())) {
                pacs.setName(pacsInfo.getPacsName());
                //           pacs.setUniqueName(pacsInfo.getPacsName());
            }
            if (StringUtil.isNotBlank(pacsInfo.getApplyDate())) {
                pacs.setDateValue(pacsInfo.getApplyDate());
            }
            pacsList.add(pacs);
        }
        return pacsList;
    }


    /**
     * @Description:病历推理
     * @Author:liuqq
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO getEnginePush(DataEngineVO dataEngineVO) {
        //基层子医院业务逻辑
        if (StringUtil.isNotEmpty(dataEngineVO.getHospitalCode()) && StringUtil.isNotEmpty(dataEngineVO.getHospitalName())) {
            //查找基层医院
            Long hospitalId = getHospitalRelation(dataEngineVO.getHospitalCode(), dataEngineVO.getHospitalName());
            if (hospitalId != null) {
                dataEngineVO.setHospitalId(hospitalId);
            } else {
                return RespDTO.onError("子医院信息异常");
            }
        } else if (dataEngineVO.getHospitalId() == null) {
            return RespDTO.onError("请输入医院id");
        } else if ("".equals(dataEngineVO.getPlanCode())) {
            return RespDTO.onError("请输入方案编码");
        }
        //初始日志记录信息
        dataEngineVO.setSubHospitalCode(dataEngineVO.getHospitalCode());
        dataEngineVO.setSubHospitalName(dataEngineVO.getHospitalName());
        dataEngineVO.setSceneName(dataEngineVO.getPlanCode());
        //dataEngineVO.setFeatureType("1,4,5,6,7,8,9,10");//推理类型

        PushVO pushVO = new PushVO();
        BeanUtil.copyProperties(dataEngineVO, pushVO);
        //增加处理后的入参
        CdssLogUtil.addBizReq(pushVO);
        PushDTO pushDTO = pushFacade.push(pushVO);
        pushDTO.setDebug(null);

        IndicationPushVO indicationPushVO = new IndicationPushVO();
        BeanUtil.copyProperties(dataEngineVO, indicationPushVO);

        //增加处理后的入参
        CdssLogUtil.addBizReq(indicationPushVO);
        IndicationDTO indicationDTO = pushFacade.indicationPush(indicationPushVO);

        // 危机值提醒
        pushDTO.setCriticalValList(indicationDTO.getCriticalValList());
        // 开单合理性提示信息
        pushDTO.setBillMsgList(indicationDTO.getBillMsgList());
        // 高危药品、手术
        pushDTO.setHighRiskList(indicationDTO.getHighRiskList());
        // 其他提醒
        pushDTO.setOtherList(indicationDTO.getOtherList());

        return RespDTO.onSuc(pushDTO);
    }

    /**
     * @Description: 开单合理性及检查、检查报告检阅
     * @Author:liuqq
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO getEngineIndicationPush(DataEngineVO dataEngineVO) {
        //基层子医院业务逻辑
        if (StringUtil.isNotEmpty(dataEngineVO.getHospitalCode()) && StringUtil.isNotEmpty(dataEngineVO.getHospitalName())) {
            //查找基层医院
            Long hospitalId = getHospitalRelation(dataEngineVO.getHospitalCode(), dataEngineVO.getHospitalName());
            if (hospitalId != null) {
                dataEngineVO.setHospitalId(hospitalId);
            } else {
                return RespDTO.onError("子医院信息异常");
            }
        } else if (dataEngineVO.getHospitalId() == null) {
            return RespDTO.onError("请输入医院id");
        } else if ("".equals(dataEngineVO.getPlanCode())) {
            return RespDTO.onError("请输入方案编码");
        }
        //初始日志记录信息
        dataEngineVO.setSubHospitalCode(dataEngineVO.getHospitalCode());
        dataEngineVO.setSubHospitalName(dataEngineVO.getHospitalName());
        dataEngineVO.setSceneName(dataEngineVO.getPlanCode());
        //dataEngineVO.setFeatureType("1,4,5,6,7,8,9,10");//推理类型
        //dataEngineVO.setRuleType("2,3,4");//推理类型（1:危急值提醒，2:开单合理项，3:高危药品、手术，4:其他提醒）

        IndicationPushVO indicationPushVO = new IndicationPushVO();
        BeanUtil.copyProperties(dataEngineVO, indicationPushVO);

        //增加处理后的入参
        CdssLogUtil.addBizReq(indicationPushVO);
        IndicationDTO indicationDTO = pushFacade.indicationPush(indicationPushVO);
        indicationDTO.setDebug(null);

        if (indicationDTO != null) {
            if (ListUtil.isNotEmpty(indicationDTO.getHighRiskList())
                    || ListUtil.isNotEmpty(indicationDTO.getCriticalValList()) || ListUtil.isNotEmpty(indicationDTO.getOtherList())) {
                return RespDTO.onSuc(indicationDTO);
            } else {
                return RespDTO.onError("无开单合理性提醒！");
            }
        } else {
            return RespDTO.onError("无开单合理性提醒！");
        }
    }


    public PushPlanDTO getOperationPlanPush(DataEngineVO dataEngineVO) {
        PushPlanVO pushPlanVO = new PushPlanVO();
        BeanUtil.copyProperties(dataEngineVO, pushPlanVO);

        PushPlanDTO pushPlanDTO = pushFacade.pushPlan(pushPlanVO);
        pushPlanDTO.setDebug(null);
        try {
            TranDatadockingLog tranDatadockingLog = new TranDatadockingLog();
            tranDatadockingLog.setHospitalId(dataEngineVO.getHospitalId());
            tranDatadockingLog.setParamIn(JSON.toJSONString(dataEngineVO));
            tranDatadockingLog.setParamOut(JSON.toJSONString(pushPlanDTO));
            tranDatadockingLog.setRemark("数据引擎模式，" + dataEngineVO.getPlanCode());

            tranDatadockingLog.setTransTime(new Date());

            dataDockingLogFacade.save(tranDatadockingLog);
        } catch (Exception e) {
            log.error("数据引擎模式，增加日志异常", e.getMessage());
        }
        return pushPlanDTO;
    }

    private Long getHospitalRelation(String hospitalCode, String hospitalName) {
        Long hospitalId = null;
        try {
            TranHospitalRelation tranHospitalRelation = new TranHospitalRelation();
            QueryWrapper<TranHospitalRelation> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("code", hospitalCode)
                    .eq("name", hospitalName)
                    .eq("is_deleted", IsDeleteEnum.N.getKey());
            tranHospitalRelation = tranHospitalRelationFacade.getOne(queryWrapper);
            if (tranHospitalRelation != null) {
                hospitalId = tranHospitalRelation.getHospitalId();
            }
        } catch (Exception e) {
            log.error("子医院信息获取异常======", e.getMessage());
            e.printStackTrace();
        }
        return hospitalId;
    }

    /**
     * @Description: 保存检查信息
     * @Author:dsYun
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO<String> sendPacsInfos(PacsVO pacsVO) {
        try {
            if (pacsVO != null) {
                //循环验证数据有效性
                if (StringUtil.isEmpty(pacsVO.getApplyNo())) {
                    return RespDTO.onError("请输入申请单号!");
                }
                //保存检查信息
                if (pacsVO.getType() == 0) {
                    TranPacsInfo pacsInfo = new TranPacsInfo();
                    BeanUtil.copyProperties(pacsVO, pacsInfo);
                    tranPacsInfoServiceImpl.save(pacsInfo);
                } else if (pacsVO.getType() == 1) { //更新检查信息删除状态
                    UpdateWrapper<TranPacsInfo> tranPacsInfo = new UpdateWrapper<>();
                    tranPacsInfo.eq("apply_no", pacsVO.getApplyNo())
                            .set("is_deleted", IsDeleteEnum.Y.getKey())
                            .set("gmt_modified", DateUtil.now());
                    tranPacsInfoServiceImpl.update(new TranPacsInfo(), tranPacsInfo);
                } else if (pacsVO.getType() == 2) { //更新修改的检查信息删除状态
                    UpdateWrapper<TranPacsInfo> tranPacsInfo = new UpdateWrapper<>();
                    tranPacsInfo.eq("apply_no", pacsVO.getApplyNo())
                            .set("pacs_name", pacsVO.getPacsName())
                            .set("results", pacsVO.getResults())
                            .set("gmt_modified", DateUtil.now());
                    tranPacsInfoServiceImpl.update(new TranPacsInfo(), tranPacsInfo);
                }

                return RespDTO.onSuc("操作正常！");
            } else {
                return RespDTO.onError("未接收到数据!");
            }
        } catch (Exception e) {
            return RespDTO.onError(e.getMessage());
        }
    }

}
