package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.diagbot.client.CdssCoreClient;
import com.diagbot.dto.*;
import com.diagbot.entity.MappingConfig;
import com.diagbot.entity.SysUser;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.MappingConfigService;
import com.diagbot.util.*;
import com.diagbot.vo.*;
import com.google.common.collect.Lists;
import lombok.val;
import net.objecthunter.exp4j.ExpressionBuilder;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2021/2/23 15:41
 */
@Component
public class KlConceptStaticFacade {

    @Autowired
    private CdssCoreClient cdssCoreClient;
    @Autowired
    private ConceptInfoFacade conceptInfoFacade;
    @Autowired
    private SysUserFacade sysUserFacade;
    @Autowired
    MrFacade mrFacade;
    @Autowired
    private AssembleFacade assembleFacade;
    @Autowired
    private MappingConfigFacade mappingConfigFacade;
    private MappingConfigService mappingConfigService;


    /**
     * 静态知识检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<StaticKnowledgeIndexDTO> staticKnowledgeIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<StaticKnowledgeIndexDTO> retList = Lists.newLinkedList();
        List<ClassicCaseIndexDTO> cases = Lists.newLinkedList();
        RespDTO<List<ClassicCaseIndexDTO>> caseDTO = RespDTO.onSuc(new ArrayList<>(0));
        RespDTO<List<StaticKnowledgeIndexDTO>> respDTO = RespDTO.onSuc(new ArrayList<>(0));

        if (staticKnowledgeIndexVO.getTypes().contains(0) ||
                staticKnowledgeIndexVO.getTypes().contains(11) ||
                staticKnowledgeIndexVO.getTypes().isEmpty()) {
            caseDTO = cdssCoreClient.classicCaseIndex(staticKnowledgeIndexVO);
            if (staticKnowledgeIndexVO.getTypes().contains(11) && staticKnowledgeIndexVO.getTypes().size() > 1)
                staticKnowledgeIndexVO.getTypes().remove(staticKnowledgeIndexVO.getTypes().indexOf(11));
        }
        if (!staticKnowledgeIndexVO.getTypes().contains(11))
            respDTO = cdssCoreClient.staticKnowledgeIndex(staticKnowledgeIndexVO);

        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
            if (RespDTOUtil.respIsOK(caseDTO)) {
                for (ClassicCaseIndexDTO classicCase : caseDTO.data) {
                    StaticKnowledgeIndexDTO dto = new StaticKnowledgeIndexDTO();
                    BeanUtil.copyProperties(classicCase, dto);
                    retList.add(dto);
                }
            }
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }

    /**
     * 经典病例检索
     *
     * @param staticKnowledgeIndexVO
     * @return
     */
    public List<ClassicCaseIndexDTO> classicCaseIndex(StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        List<ClassicCaseIndexDTO> retList = Lists.newLinkedList();

        RespDTO<List<ClassicCaseIndexDTO>> respDTO = cdssCoreClient.classicCaseIndex(staticKnowledgeIndexVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            retList = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }

        if (staticKnowledgeIndexVO.getHasInfo() != null && staticKnowledgeIndexVO.getHasInfo().equals(1)) {
            retList = retList.stream().filter(i -> i.getStatus().equals(1)).collect(Collectors.toList());
        }
        return retList;
    }

    /**
     * 获取静态知识
     *
     * @param staticKnowledgeVO
     * @return
     */
    public StaticKnowledgeDTO getStaticKnowledge(StaticKnowledgeVO staticKnowledgeVO) {
        ScaleStaticAllVO scaleStaticAllVO = new ScaleStaticAllVO();
        if (staticKnowledgeVO.getType() == 8) {
            PushJoinVO mr = mrFacade.getMr(staticKnowledgeVO.getMrId());
            if (null != mr) {
                SearchData searchData = new SearchData();
                BeanUtil.copyProperties(mr, searchData);
                //入参映射
                searchData = assembleFacade.assembleData(searchData);
                BeanUtil.copyProperties(searchData, scaleStaticAllVO);
            } else {
                scaleStaticAllVO.setHospitalId(-1L);
            }
        } else {
            scaleStaticAllVO.setHospitalId(-1L);
        }
        BeanUtil.copyProperties(staticKnowledgeVO, scaleStaticAllVO);
        RespDTO<StaticKnowledgeDTO> respDTO;
        if (staticKnowledgeVO.getType().equals(11))
            respDTO = cdssCoreClient.getClassicCaseInfo(scaleStaticAllVO);
        else
            respDTO = cdssCoreClient.getStaticKnowledge(scaleStaticAllVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            StaticKnowledgeDTO staticKnowledgeDTO = respDTO.data;
            return staticKnowledgeDTO;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
    }

    /**
     * 获取量表计算公式结果
     */
    public CalculatorFormulaScoreDTO getCalculatorFormulaResult(CalculatorFormulaVO calculatorFormulaVO) {

        // 1. 获取计算公式配置
        KlCalculatorFormulaDTO formulaConfig = getFormulaConfig(calculatorFormulaVO.getConceptId());

        // 2. 参数预处理
        Map<String, Object> inputParams = Optional.ofNullable(calculatorFormulaVO.getParams())
                .orElseGet(HashMap::new);

        // 3. 匹配计算公式
        String matchedFormula = matchFormula(formulaConfig, inputParams);

        // 4. 转换计算参数
        Map<String, Double> calcParams = convertParams(inputParams);

        // 5. 执行计算并返回结果
        String score = calculateResult(matchedFormula, calcParams);

        // 6. 比较得分，并返回响应的结果
        String result = getResult(formulaConfig, matchedFormula, score);

        CalculatorFormulaScoreDTO scoreDTO = new CalculatorFormulaScoreDTO();
        scoreDTO.setScore(score);
        scoreDTO.setResult(result);
        return scoreDTO;
    }

    private String getResult(KlCalculatorFormulaDTO formulaConfig, String matchedFormula, String score) {
        String result = null;
        KlCalculatorFormulaDTO.ScoreTableData scoreTableData = formulaConfig.getScoreTableData().stream().filter(e -> e.getFormula().equalsIgnoreCase(matchedFormula)).findFirst().get();
        List<KlCalculatorFormulaDTO.ScoreRange> scoreRanges = scoreTableData.getScoreRange();
        // 6. 匹配得分范围
        double scoreValue;
        try {
            scoreValue = Double.parseDouble(score);
        } catch (NumberFormatException e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "得分格式错误: " + score);
        }

        for (KlCalculatorFormulaDTO.ScoreRange range : scoreRanges) {
            BigDecimal ge = range.getGreaterEqual();  // 下限
            BigDecimal le = range.getLessEqual();     // 上限

            // 边界条件检查
            boolean lowerBoundMet = ge == null || scoreValue >= ge.doubleValue();
            boolean upperBoundMet = le == null || scoreValue <= le.doubleValue();

            // 特殊场景处理
            if (ge != null && le != null) {
                // 当配置的上下限反序时自动校正
                if (ge.doubleValue() > le.doubleValue()) {
                    lowerBoundMet = scoreValue >= le.doubleValue();
                    upperBoundMet = scoreValue <= ge.doubleValue();
                }
            }

            // 匹配条件判断
            if (lowerBoundMet && upperBoundMet) {
                result = range.getResult();
                break; // 找到第一个匹配项即返回
            }
        }

        // 后置校验
        if (result == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "得分" + score + "未匹配到有效范围");
        }
        return result;
    }

    /**
     * 获取计算公式配置
     */
    private KlCalculatorFormulaDTO getFormulaConfig(Long conceptId) {
        return Optional.ofNullable(cdssCoreClient.getRecordById(new IdVO(conceptId)))
                .filter(RespDTOUtil::respIsOK)
                .map(resp -> resp.data.getCalculatorFormula())
                .orElseThrow(() -> new CommonException(CommonErrorCode.SERVER_IS_ERROR, "计算公式配置不存在"));
    }

    /**
     * 匹配计算公式
     */
    private String matchFormula(KlCalculatorFormulaDTO formulaConfig, Map<String, Object> params) {
        return formulaConfig.getScoreTableData().stream()
                .collect(Collectors.groupingBy(KlCalculatorFormulaDTO.ScoreTableData::getGroupId))
                .values().stream()
                .filter(group -> checkGroupConditions(group, params))
                .findFirst()
                .map(group -> group.get(0).getFormula())
                .orElseThrow(() -> new CommonException(CommonErrorCode.SERVER_IS_ERROR, "无匹配计算公式"));
    }

    /**
     * 检查组条件
     */
    private boolean checkGroupConditions(List<KlCalculatorFormulaDTO.ScoreTableData> group,
                                         Map<String, Object> params) {
        return group.stream()
                .allMatch(tableData -> checkSingleCondition(tableData, params));
    }


    /**
     * 检查单个条件
     */
    private boolean checkSingleCondition(KlCalculatorFormulaDTO.ScoreTableData tableData,
                                         Map<String, Object> params) {
        String conditionName = tableData.getConditionName();
        Object inputValue = params.get(conditionName);

        if (inputValue == null) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "缺少必要参数: " + conditionName);
        }

        return Optional.ofNullable(tableData.getConditionType())
                .map(type -> {
                    if ("数值类型".equals(type)) {
                        return checkNumericCondition(inputValue, tableData.getConditionContent().getValueType());
                    } else if ("文本类型".equals(type)) {
                        return checkTextCondition(inputValue, tableData.getConditionContent().getTextType());
                    }
                    throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "未知条件类型: " + type);
                })
                .orElseThrow(() -> new CommonException(CommonErrorCode.SERVER_IS_ERROR, "未指定条件类型"));
    }

    /**
     * 数值条件校验（使用JDK8的Function接口）
     */
    private boolean checkNumericCondition(Object inputValue, KlCalculatorFormulaDTO.ValueType condition) {
        // 获取最小值校验函数（已绑定 threshold）
        Function<Double, Boolean> minCheck = getNumericCheckFunction(
                condition.getMinValueName(),
                condition.getMinValue() // 直接传递阈值
        );

        // 获取最大值校验函数（已绑定 threshold）
        Function<Double, Boolean> maxCheck = getNumericCheckFunction(
                condition.getMaxValueName(),
                condition.getMaxValue() // 直接传递阈值
        );

        try {
            double value = Double.parseDouble(inputValue.toString());
            return minCheck.apply(value) && maxCheck.apply(value);
        } catch (NumberFormatException e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "参数类型错误: " + inputValue);
        }
    }

    /**
     * 生成数值校验函数
     */
    private Function<Double, Boolean> getNumericCheckFunction(String operator, BigDecimal threshold) {
        if (operator == null || threshold == null) {
            return value -> true; // 默认通过校验
        }

        double thresholdValue = threshold.doubleValue();
        switch (operator) {
            case ">":  return value -> value > thresholdValue;
            case ">=": return value -> value >= thresholdValue;
            case "<":  return value -> value < thresholdValue;
            case "<=": return value -> value <= thresholdValue;
            default:
                throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "无效运算符: " + operator);
        }
    }

    /**
     * 文本条件校验
     */
    private boolean checkTextCondition(Object inputValue, String expectedText) {
        return Optional.ofNullable(expectedText)
                .map(t -> t.equals(inputValue.toString()))
                .orElseThrow(() -> new CommonException(CommonErrorCode.SERVER_IS_ERROR, "文本条件未配置"));
    }

    /**
     * 参数转换（使用Stream API）
     */
    private Map<String, Double> convertParams(Map<String, Object> params) {
        return params.entrySet().stream()
                .map(entry -> {
                    Optional<Double> valueOpt = tryParseDouble(entry.getValue());
                    return valueOpt.map(v -> new AbstractMap.SimpleEntry<>(entry.getKey(), v));
                })
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toMap(
                        Map.Entry::getKey,
                        Map.Entry::getValue,
                        (existing, replacement) -> existing // 处理重复key的情况
                ));
    }

    /**
     * 安全转换数值方法（返回Optional）
     */
    private Optional<Double> tryParseDouble(Object value) {
        if (value == null) return Optional.empty();

        try {
            return Optional.of(Double.parseDouble(value.toString()));
        } catch (NumberFormatException e) {
            return Optional.empty();
        }
    }

    /**
     * 执行计算
     */
    private String calculateResult(String formula, Map<String, Double> params) {
        try {
            double result = new ExpressionBuilder(formula)
                    .variables(params.keySet())
                    .build()
                    .setVariables(params)
                    .evaluate();
            return formatDouble(result);
        } catch (IllegalArgumentException | ArithmeticException e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "公式计算失败: " + e.getMessage());
        }
    }

    /**
     * 结果格式化
     */
    private String formatDouble(double value) {
        return BigDecimal.valueOf(value)
                .setScale(6, RoundingMode.HALF_UP)
                .stripTrailingZeros()
                .toPlainString();
    }

    /**
     * 安全数值转换
     */
    private Double convertToDouble(String paramName, Object value) {
        try {
            return Double.parseDouble(value.toString());
        } catch (NumberFormatException e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
                    "参数[" + paramName + "]转换失败: " + value);
        }
    }



    /**
     * 医院端获取静态知识（对接）
     *
     * @param staticKnowledgeHISVO
     * @return
     */
    public List<StaticKnowledgeHISDTO> getStaticKnowledgeForHIS(StaticKnowledgeHISVO staticKnowledgeHISVO) {

        List<StaticKnowledgeHISDTO> retList = Lists.newArrayList();

        //术语映射
        List<String> uniqueNameList = conceptInfoFacade.getUniqueNames(staticKnowledgeHISVO);
        if (ListUtil.isEmpty(uniqueNameList)) {
            return retList;
        }

        for (String uniqueName : uniqueNameList) {
            StaticKnowledgeVO staticKnowledgeVO = new StaticKnowledgeVO();
            BeanUtil.copyProperties(staticKnowledgeHISVO, staticKnowledgeVO);
            staticKnowledgeVO.setName(uniqueName);
            try {
                StaticKnowledgeDTO staticKnowledgeDTO = getStaticKnowledge(staticKnowledgeVO);
                if (staticKnowledgeDTO != null) {
                    StaticKnowledgeHISDTO staticKnowledgeHISDTO = new StaticKnowledgeHISDTO();
                    BeanUtil.copyProperties(staticKnowledgeDTO, staticKnowledgeHISDTO);
                    staticKnowledgeHISDTO.setHisName(staticKnowledgeHISVO.getHisName());
                    staticKnowledgeHISDTO.setHisDetailName(staticKnowledgeHISVO.getHisDetailName());
                    retList.add(staticKnowledgeHISDTO);
                }
            } catch (CommonException e) {
            }
        }
        return retList;
    }

    /**
     * 医院对接静态知识判断是否有静态知识
     *
     * @param staticKnowledgeHISVO
     * @return
     */
    public Boolean isExistForHIS(StaticKnowledgeHISVO staticKnowledgeHISVO) {
        List<StaticKnowledgeHISDTO> staticList = getStaticKnowledgeForHIS(staticKnowledgeHISVO);
        if (ListUtil.isEmpty(staticList)) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * 分页查询
     *
     * @param klConceptStaticPageVO
     * @return
     */
    public IPage<KlConceptStaticDTO> getPage(KlConceptStaticPageVO klConceptStaticPageVO) {
        Page<KlConceptStaticDTO> page = null;
        RespDTO<Page<KlConceptStaticDTO>> respDTO = cdssCoreClient.getPage(klConceptStaticPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        List<KlConceptStaticDTO> records = page.getRecords();
        List<Long> userIds = records.stream().map(KlConceptStaticDTO::getModifier)
                .distinct()
                .map(Long::parseLong)
                .collect(Collectors.toList());
        List<SysUser> users = sysUserFacade.list(new QueryWrapper<SysUser>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("id", userIds, false));
        if (ListUtil.isNotEmpty(users)) {
            Map<Long, String> userMap = users.stream().collect(Collectors.toMap(SysUser::getId, SysUser::getLinkman));
            records.forEach(record -> {
                if (userMap.containsKey(Long.valueOf(record.getModifier()))) {
                    record.setModifier(userMap.get(Long.valueOf(record.getModifier())));
                } else {
                    record.setModifier("");
                }
            });

        } else {
            records.forEach(record -> {
                record.setModifier("");
            });
        }
        page.setRecords(records);
        return page;
    }

    /**
     * 经典病例分页查询
     *
     * @param klClassicCasePageVO
     * @return
     */
    public IPage<KlClassicCaseDTO> getClassicCase(KlClassicCasePageVO klClassicCasePageVO) {
        Page<KlClassicCaseDTO> page = null;
        RespDTO<Page<KlClassicCaseDTO>> respDTO = cdssCoreClient.getClassicCase(klClassicCasePageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        List<KlClassicCaseDTO> records = page.getRecords();
        List<String> userIds = records.stream().map(KlClassicCaseDTO::getModifier)
                .distinct()
                .collect(Collectors.toList());
        RespDTO<Map<String, String>> userRespDTO = RespDTO.onSuc(sysUserFacade.getUserInfoByIds(userIds));
        if (RespDTOUtil.respIsNG(userRespDTO)) {
            return page;
        }
        Map<String, String> userMap = userRespDTO.data;
        records.forEach(record -> {
            if (userMap.containsKey(record.getModifier())) {
                record.setModifier(userMap.get(record.getModifier()));
            } else {
                record.setModifier("");
            }
        });
        page.setRecords(records);
        return page;
    }

    /**
     * 分页查询
     *
     * @param staticKnowledgeIndexPageVO
     * @return
     */
    public IPage<StaticKnowledgeIndexPageDTO> staticIndexPage(StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        Page<StaticKnowledgeIndexPageDTO> page = null;

        RespDTO<Page<StaticKnowledgeIndexPageDTO>> respDTO = cdssCoreClient.staticIndexPage(staticKnowledgeIndexPageVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            page = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return page;
    }

    /**
     * 保存记录（新增or修改）
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean saveOrUpdateRecord(KlConceptStaticVO klConceptStaticVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klConceptStaticVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateRecord(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 保存病例（新增or修改）
     *
     * @param klClassicCaseVO
     * @return
     */
    public Boolean saveOrUpdateCase(KlClassicCaseVO klClassicCaseVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        klClassicCaseVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.saveOrUpdateCase(klClassicCaseVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 经典病例启用禁用
     *
     * @param changeStatusVO
     * @return
     */
    public Boolean changeCaseStatus(ChangeStatusVO changeStatusVO) {
        String userId = UserUtils.getCurrentPrincipleID();
        changeStatusVO.setUserId(Long.valueOf(userId));

        Boolean success = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.changeCaseStatus(changeStatusVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            success = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return success;
    }

    /**
     * 根据术语id获取静态信息
     *
     * @param idVO
     * @return
     */
    public KlConceptStaticDTO getRecordById(IdVO idVO) {
        KlConceptStaticDTO klConceptStaticDTO = new KlConceptStaticDTO();
        RespDTO<KlConceptStaticDTO> respDTO = cdssCoreClient.getRecordById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klConceptStaticDTO = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return klConceptStaticDTO;
    }

    /**
     * 根据经典病例id获取病例信息
     *
     * @param idVO
     * @return
     */
    public KlClassicCaseDTO getCaseById(IdVO idVO) {
        KlClassicCaseDTO klClassicCaseDTO = new KlClassicCaseDTO();
        RespDTO<KlClassicCaseDTO> respDTO = cdssCoreClient.getCaseById(idVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            klClassicCaseDTO = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return klClassicCaseDTO;
    }

    /**
     * 是否已存在
     *
     * @param klConceptStaticVO
     * @return
     */
    public Boolean isExist(KlConceptStaticVO klConceptStaticVO) {
        Boolean exist = false;
        RespDTO<Boolean> respDTO = cdssCoreClient.isExist(klConceptStaticVO);
        if (RespDTOUtil.respIsOK(respDTO)) {
            exist = respDTO.data;
        } else {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, respDTO.msg);
        }
        return exist;
    }
}