package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.RespDTO;
import com.diagbot.entity.HospitalInfo;
import com.diagbot.entity.SysUser;
import com.diagbot.entity.SysUserHospital;
import com.diagbot.entity.SysUserRole;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.StatusEnum;
import com.diagbot.vo.UserInfoVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.crypto.factory.PasswordEncoderFactories;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;

/**
 * @Description:用户维护接口
 * @Author:liuqq
 * @time: ${DATE} ${TIME}
 **/
@Component
@Slf4j
public class UserDockFacade {

    @Autowired
    private SysUserFacade sysUserFacade;
    @Autowired
    private SysUserRoleFacade sysUserRoleFacade;
    @Autowired
    private SysUserHospitalFacade sysUserHospitalFacade;

    //private

    /**
     * @Description:添加用户，初始化用户角色为-1
     * @Author:liuqq
     * @time: ${DATE} ${TIME}
     **/
    public RespDTO addUser(UserInfoVO userInfoVO){
        try{
            QueryWrapper<SysUser> queryWrapper = new QueryWrapper<SysUser>();
            queryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                    .eq("status", StatusEnum.Enable.getKey())
                    .eq("username", userInfoVO.getUsername());
            SysUser user= sysUserFacade.getOne(queryWrapper);

            if(user !=null){
                SysUser sysUser=new SysUser();
                BeanUtils.copyProperties(userInfoVO,sysUser);
                sysUser.setId(user.getId());
                PasswordEncoder passwordEncoder= PasswordEncoderFactories.createDelegatingPasswordEncoder();
                sysUser.setPassword(passwordEncoder.encode(userInfoVO.getPassword()));
                sysUserFacade.updateById(sysUser);
            }else{
                SysUser sysUser=new SysUser();
                BeanUtils.copyProperties(userInfoVO,sysUser);
                PasswordEncoder passwordEncoder= PasswordEncoderFactories.createDelegatingPasswordEncoder();
                sysUser.setPassword(passwordEncoder.encode(userInfoVO.getPassword()));
                sysUserFacade.save(sysUser);

                user= sysUserFacade.getOne(queryWrapper);
                SysUserRole sysUserRole=new SysUserRole();
                sysUserRole.setUserId(user.getId());
                sysUserRole.setRoleId(Long.valueOf("-1"));
                sysUserRoleFacade.save(sysUserRole);

                SysUserHospital sysUserHospital=new SysUserHospital();
                sysUserHospital.setUserId(user.getId());
                sysUserHospital.setHospitalId(userInfoVO.getHospitalId());
                sysUserHospitalFacade.save(sysUserHospital);
            }

            return RespDTO.onSuc("操作正常");
        }catch (Exception e){
            e.printStackTrace();
            return RespDTO.onError("操作异常");
        }
    }
}
