package com.diagbot.vo;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.biz.push.entity.Scale;
import io.swagger.annotations.ApiModelProperty;
import lombok.Getter;
import lombok.Setter;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Getter
@Setter
public class DataEngineVO extends CdssLogBaseVO{

    @NotNull(message = "请输入方案编码")
    private String planCode;//方案编码，用于控制哪些展示项

    /**
     * 推理类型（1：症状，4：查体结果，5：检验，6：检查，7：诊断，8：药品，9：手术）
     */
    private String featureType = "";

    /**
     * 规则类型（1:危急值提醒，2:开单合理项，3:高危药品、手术，4:其他提醒）
     */
    private String ruleType = "";

    /**
     * 是否获取静态知识（0：不获取，1：获取）
     */
    @ApiModelProperty(hidden = true)
    private Integer getStaticKnowledge = 0;

    /**
     * 医院编码
     */
    private String hospitalCode;
    /**
     * 医院名称
     */
    private String hospitalName;

    /**
     * 年龄文本数据
     */
    private String age;

    /**
     * 性别（1：男，2：女，3：通用）
     */
    private Integer sex;
    /**
     * 婚姻
     */
    private String marriage = "";
    /**
     * 主诉
     */
    private String chief = "";
    /**
     * 现病史
     */
    private String symptom = "";
    /**
     * 查体
     */
    private String vital = "";
    /**
     * 既往史
     */
    private String pasts = "";
    /**
     * 传染病史
     */
    private String infectious = "";
    /**
     * 手术外伤史
     */
    private String surgical = "";
    /**
     * 过敏史
     */
    private String allergy = "";
    /**
     * 接种史
     */
    private String vaccination = "";
    /**
     * 个人史
     */
    private String personal = "";
    /**
     * 婚育史
     */
    private String marital = "";
    /**
     * 家族史
     */
    private String family = "";
    /**
     * 月经史
     */
    private String menstrual = "";
    /**
     * 其他史
     */
    private String other = "";
    /**
     * 化验文本数据
     */
    private String lisString = "";
    /**
     * 辅检文本数据
     */
    private String pacsString = "";
    /**
     * 诊断文本数据
     */
    private String diagString = "";
    /**
     * 药品文本数据
     */
    private String drugString = "";
    /**
     * 科室
     */
    private List<Item> dept = new ArrayList<>();
    /**
     * 化验项目和结果
     */
    private List<Lis> lis = new ArrayList<>();
    /**
     * 辅检项目和结果
     */
    private List<Pacs> pacs = new ArrayList<>();
    /**
     * 诊断
     */
    private List<Item> diag = new ArrayList<>();
    /**
     * 药品
     */
    private List<Drug> drug = new ArrayList<>();
    /**
     * 手术及操作
     */
    private List<Item> operation = new ArrayList<>();
    /**
     * 输血记录
     */
    private List<ItemExt> transfusion = new ArrayList<>();
    /**
     * 当前化验开单项
     */
    private List<Lis> lisOrder = new ArrayList<>();
    /**
     * 当前辅检开单项
     */
    private List<Pacs> pacsOrder = new ArrayList<>();
    /**
     * 当前诊断开单项
     */
    private List<Item> diagOrder = new ArrayList<>();
    /**
     * 当前药品开单项
     */
    private List<Drug> drugOrder = new ArrayList<>();
    /**
     * 当前手术开单项
     */
    private List<Item> operationOrder = new ArrayList<>();
    /**
     * 当前输血开单项
     */
    private List<ItemExt> transfusionOrder = new ArrayList<>();
    /**
     * 其他开单项
     */
    private List<Item> otherOrder = new ArrayList<>();
    /**
     * 选中诊断
     */
    private Item diseaseName;
    /**
     * 选中手术
     */
    private Item operationName;
    /**
     * 其他指标
     */
    private Map<String, String> otherIndex;
    /**
     * 其他医嘱
     */
    private List<Item> otherAdvice = new ArrayList<>();
    /**
     * 量表结果
     */
    private List<Scale> scale = new ArrayList<>();

    /**
     * 医生
     */
    private Doctor doctor;
}
