package com.diagbot.web;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.annotation.SysLogger;
import com.diagbot.dto.*;
import com.diagbot.facade.ConceptInfoFacade;
import com.diagbot.facade.KlConceptStaticFacade;
import com.diagbot.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import jdk.nashorn.internal.ir.annotations.Ignore;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;

/**
 * <p>
 * 静态知识标准术语表 前端控制器
 * </p>
 *
 * @author zhaops
 * @since 2020-08-18
 */
@RestController
@RequestMapping("/kl/conceptInfo")
@Api(value = "静态知识标准术语相关API", tags = { "静态知识标准术语相关API" })
@SuppressWarnings("unchecked")
public class ConceptInfoController {

    @Autowired
    private ConceptInfoFacade conceptInfoFacade;

    @Autowired
    private KlConceptStaticFacade klConceptStaticFacade;

    @ApiOperation(value = "医学知识（静态知识）检索[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndex")
    @SysLogger("staticKnowledgeIndex")
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndex(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        staticKnowledgeIndexVO.setHasInfo(1);
        List<StaticKnowledgeIndexDTO> data = klConceptStaticFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医学术语检索-新增静态知识[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticKnowledgeIndexWithoutInfo")
    @SysLogger("staticKnowledgeIndexWithoutInfo")
    @Transactional
    public RespDTO<List<StaticKnowledgeIndexDTO>> staticKnowledgeIndexWithoutInfo(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        staticKnowledgeIndexVO.setHasInfo(0);
        List<StaticKnowledgeIndexDTO> data = klConceptStaticFacade.staticKnowledgeIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "医学术语检索-新增经典病例",
            notes = "types： 类型：11-经典病例 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/classicCaseIndexWithoutInfo")
    @SysLogger("classicCaseIndexWithoutInfo")
    public RespDTO<List<ClassicCaseIndexDTO>> classicCaseIndexWithoutInfo(@Valid @RequestBody StaticKnowledgeIndexVO staticKnowledgeIndexVO) {
        staticKnowledgeIndexVO.setHasInfo(0);
        List<ClassicCaseIndexDTO> data = klConceptStaticFacade.classicCaseIndex(staticKnowledgeIndexVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "页面获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案、5-诊疗指南<br>" +
                    "name： 标准术语名称<br>")
    @PostMapping("/getStaticKnowledge")
    @SysLogger("getStaticKnowledge")
    public RespDTO<StaticKnowledgeDTO> getStaticKnowledge(@Valid @RequestBody StaticKnowledgeVO staticKnowledgeVO) {
        StaticKnowledgeDTO data = klConceptStaticFacade.getStaticKnowledge(staticKnowledgeVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "页面获取量表计算公式结果")
    @PostMapping("/getCalculatorFormulaResult")
    @SysLogger("getCalculatorFormulaResult")
    public RespDTO<CalculatorFormulaScoreDTO> getCalculatorFormulaResult(@Valid @RequestBody CalculatorFormulaVO calculatorFormulaVO) {
        CalculatorFormulaScoreDTO data = klConceptStaticFacade.getCalculatorFormulaResult(calculatorFormulaVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "对接获取静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案、5-诊疗指南 <br>" +
                    "hisName： HIS大项名称<br>" +
                    "hisDetailName： HIS小项名称<br>")
    @PostMapping("/getStaticKnowledgeForHIS")
    @SysLogger("getStaticKnowledgeForHIS")
    public RespDTO<List<StaticKnowledgeHISDTO>> getStaticKnowledgeForHIS(@Valid @RequestBody StaticKnowledgeHISVO staticKnowledgeHISVO) {
        List<StaticKnowledgeHISDTO> data = klConceptStaticFacade.getStaticKnowledgeForHIS(staticKnowledgeHISVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "对接获取静态知识-判断是否有静态知识[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "contentTypes： 内容类型（多选）：1-静态信息、2-注意事项、3-临床路径、4-治疗方案、5-诊疗指南 <br>" +
                    "hisName： HIS大项名称<br>" +
                    "hisDetailName： HIS小项名称<br>")
    @PostMapping("/isExistForHIS")
    @SysLogger("isExistForHIS")
    public RespDTO<Boolean> isExistForHIS(@Valid @RequestBody StaticKnowledgeHISVO staticKnowledgeHISVO) {
        Boolean data = klConceptStaticFacade.isExistForHIS(staticKnowledgeHISVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取静态知识列表[zhaops]",
            notes = "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "name： 术语名称<br>" +
                    "isDeleted： 启用状态：N-启用中、Y-已删除<br>")
    @PostMapping("/getPage")
    @SysLogger("getPage")
    @SuppressWarnings("unchecked")
    public RespDTO<IPage<KlConceptStaticDTO>> getPage(@Valid @RequestBody KlConceptStaticPageVO klConceptStaticPageVO) {
        IPage<KlConceptStaticDTO> data = klConceptStaticFacade.getPage(klConceptStaticPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "获取经典病例列表",
            notes = "type： 类型：11-经典病例 <br>" +
                    "name： 术语名称<br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/getClassicCase")
    @SysLogger("getClassicCase")
    public RespDTO<IPage<KlClassicCaseDTO>> getClassicCase(@Valid @RequestBody KlClassicCasePageVO klClassicCasePageVO) {
        IPage<KlClassicCaseDTO> data = klConceptStaticFacade.getClassicCase(klClassicCasePageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识检索（分页返回）[zhaops]",
            notes = "types： 类型（多选）：0-全部、1-诊断、2-药品、3-检验、5-检查、6-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "inputStr： 检索内容<br>")
    @PostMapping("/staticIndexPage")
    @SysLogger("staticIndexPage")
    @SuppressWarnings("unchecked")
    public RespDTO<IPage<StaticKnowledgeIndexPageDTO>> staticIndexPage(@Valid @RequestBody StaticKnowledgeIndexPageVO staticKnowledgeIndexPageVO) {
        IPage<StaticKnowledgeIndexPageDTO> data = klConceptStaticFacade.staticIndexPage(staticKnowledgeIndexPageVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存静态知识-新增或修改[zhaops]",
            notes = "id： id <br>" +
                    "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规 <br>" +
                    "clinicalPathwayName： 临床路径名称<br>" +
                    "noticeName： 注意事项名称<br>" +
                    "details： 明细<br>")
    @PostMapping("/saveOrUpdateRecord")
    @SysLogger("saveOrUpdateRecord")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateRecord(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.saveOrUpdateRecord(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "保存经典病例-新增或修改",
            notes = "id： id <br>" +
                    "name： 术语名称 <br>" +
                    "type： 类型：11-经典病例  <br>" +
                    "source：数据来源：0-医院端、1-云平台，默认0 <br>" +
                    "details： 明细<br>" +
                    "scale：评估量表<br>")
    @PostMapping("/saveOrUpdateCase")
    @SysLogger("saveOrUpdateCase")
    @Transactional
    public RespDTO<Boolean> saveOrUpdateCase(@Valid @RequestBody KlClassicCaseVO klClassicCaseVO) {
        Boolean data = klConceptStaticFacade.saveOrUpdateCase(klClassicCaseVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识启用禁用[zhaops]",
            notes = "id： 术语id <br>" +
                    "status： 启用状态（0：禁用，1：启用）<br>")
    @PostMapping("/changeStatus")
    @SysLogger("changeStatus")
    @Transactional
    public RespDTO<Boolean> changeStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = klConceptStaticFacade.changeStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "经典病例启用禁用",
            notes = "id： 术语id <br>" +
                    "status： 启用状态：1-启用、0-禁用<br>")
    @PostMapping("/changeCaseStatus")
    @SysLogger("changeCaseStatus")
    @Transactional
    public RespDTO<Boolean> changeCaseStatus(@Valid @RequestBody ChangeStatusVO changeStatusVO) {
        Boolean data = klConceptStaticFacade.changeCaseStatus(changeStatusVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "静态知识是否存在[zhaops]",
            notes = "name： 术语名称 <br>" +
                    "type： 类型：1-诊断、2-药品、3-检验套餐、4-检验细项、5-检查、6-检查子项、7-手术和操作、8-量表、9-护理、10-政策法规 <br>")
    @PostMapping("/isExist")
    @SysLogger("isExist")
    public RespDTO<Boolean> isExist(@Valid @RequestBody KlConceptStaticVO klConceptStaticVO) {
        Boolean data = klConceptStaticFacade.isExist(klConceptStaticVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据术语id获取静态信息[zhaops]",
            notes = "id： 术语id <br>")
    @PostMapping("/getRecordById")
    @SysLogger("getRecordById")
    public RespDTO<KlConceptStaticDTO> getRecordById(@Valid @RequestBody IdVO idVO) {
        KlConceptStaticDTO data = klConceptStaticFacade.getRecordById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "根据病例id获取病例信息",
            notes = "id： 病例id <br>")
    @PostMapping("/getCaseById")
    @SysLogger("getCaseById")
    public RespDTO<KlClassicCaseDTO> getCaseById(@Valid @RequestBody IdVO idVO) {
        KlClassicCaseDTO data = klConceptStaticFacade.getCaseById(idVO);
        return RespDTO.onSuc(data);
    }

    @ApiOperation(value = "批量更新静态知识标志[zhaops]",
            notes = "")
    @PostMapping("/updateHasInfoStatusBatch")
    @SysLogger("updateHasInfoStatusBatch")
    @Ignore
    public RespDTO<Boolean> updateHasInfoStatusBatch() {
        Boolean data = conceptInfoFacade.updateHasInfoStatusBatch();
        return RespDTO.onSuc(data);
    }
}